@@ -3,17 +3,15 @@
  * The IMP_Folder:: class provides a set of methods for dealing with folders,
  * accounting for subscription, errors, etc.
  *
- * $Horde: imp/lib/Folder.php,v 1.130.10.43 2007/07/18 20:09:50 chuck Exp $
+ * $Horde: imp/lib/Folder.php,v 1.130.10.48 2008/08/08 05:07:52 slusarz Exp $
  *
- * Copyright 2000-2007 Chuck Hagenbuch <chuck@horde.org>
- * Copyright 2000-2007 Jon Parise <jon@csh.rit.edu>
+ * Copyright 2000-2008 The Horde Project (http://www.horde.org/)
  *
- * See the enclosed file COPYING for license information (GPL). If you did not
- * receive this file, see http://www.fsf.org/copyleft/gpl.html.
+ * See the enclosed file COPYING for license information (GPL). If you
+ * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Jon Parise <jon@csh.rit.edu>
- * @since   IMP 2.3
  * @package IMP
  */
 class IMP_Folder {
@@ -26,21 +24,6 @@ class IMP_Folder {
      */
     var $_listCache = array();
 
-    /**
-     * Keep track of mailbox names that we have complained about to prevent
-     * giving the user identical error messages.
-     *
-     * @var array
-     */
-    var $_errorCache = array();
-
-    /**
-     * Cached results from the exists() function.
-     *
-     * @var array
-     */
-    var $_existsResults = array();
-
     /**
      * Returns a reference to the global IMP_Folder object, only creating it
      * if it doesn't already exist. This ensures that only one IMP_Folder
@@ -111,59 +94,27 @@ class IMP_Folder {
             }
         }
 
-        if (isset($this->_listCache[$subidx])) {
-            $maildelim = $this->_listCache[$subidx]['d'];
-            $mailsort = $this->_listCache[$subidx]['s'];
-        } else {
-            $maildelim = $mailsort = array();
-            foreach ($this->_listFolders($sub) as $val) {
-                /* Strip off the prefix only if we are dealing with a personal
-                 * namespace. */
-                $label = $stripped = substr($val, strpos($val, '}') + 1);
-                $ns_info = IMP::getNamespace($label);
-                if ($ns_info['type'] == 'personal') {
-                    $label = substr($label, strlen($ns_info['name']));
-                }
-                if (strcasecmp('INBOX', $label) == 0) {
-                    continue;
-                }
-                $maildelim[$stripped] = $ns_info['delimiter'];
-                $mailsort[$stripped] = $label;
+        require_once IMP_BASE . '/lib/IMAP/Tree.php';
+        $imaptree = &IMP_Tree::singleton();
+
+        if (!isset($this->_listCache[$subidx])) {
+            $list_mask = IMPTREE_FLIST_CONTAINER | IMPTREE_FLIST_OB;
+            if (!$sub) {
+                $list_mask |= IMPTREE_FLIST_UNSUB;
             }
-            $this->_listCache[$subidx] = array('d' => $maildelim, 's' => $mailsort);
+            $this->_listCache[$subidx] = $imaptree->folderList($list_mask);
         }
 
-        if (!empty($mailsort)) {
-            // TODO: Fix use of delimiter here
-            require_once IMP_BASE . '/lib/IMAP/Sort.php';
-            $delimiter = reset($_SESSION['imp']['namespace']);
-            $imap_sort = &new IMP_IMAP_Sort($delimiter['delimiter']);
-            $imap_sort->sortMailboxes($mailsort, true, true);
-
-            foreach ($mailsort as $mbox_name => $label) {
-                if (in_array($mbox_name, $filter)) {
-                    continue;
-                }
-
-                if (!($decoded_mailbox = String::convertCharset($mbox_name, 'UTF7-IMAP')) &&
-                    empty($this->_errorCache[$mbox_name])) {
-                    $notification->push(sprintf(_("The folder \"%s\" contains illegal characters in its name. It may cause problems. Please see your system administrator."), $label), 'horde.warning');
-                    $this->_errorCache[$mbox_name] = true;
-                }
-
-                $parts = ($maildelim[$mbox_name]) ? explode($maildelim[$mbox_name], $label) : array($label);
-                $partcount = count($parts);
-                for ($i = 1; $i <= $partcount; $i++) {
-                    $item = implode($maildelim[$mbox_name], array_slice($parts, 0, $i));
-                    if (!isset($list[$item])) {
-                        $abbrev = $folded = str_repeat(' ', 4 * ($i - 1)) . String::convertCharset($parts[($i - 1)], 'UTF7-IMAP');
-                        if (strlen($abbrev) > 26) {
-                            $abbrev = String::substr($abbrev, 0, 10) . '...' . String::substr($abbrev, -13, 13);
-                        }
-                        $list[$item] = array('val' => ($i == $partcount) ? $mbox_name : '', 'label' => $folded, 'abbrev' => $abbrev);
-                    }
-                }
+        foreach ($this->_listCache[$subidx] as $ob) {
+            if (in_array($ob['v'], $filter)) {
+                continue;
             }
+
+            $abbrev = $label = str_repeat(' ', 4 * $ob['c']) . $ob['l'];
+            if (strlen($abbrev) > 26) {
+                $abbrev = String::substr($abbrev, 0, 10) . '...' . String::substr($abbrev, -13, 13);
+            }
+            $list[$ob['v']] = array('val' => $imaptree->isContainer($ob) ? '' : $ob['v'], 'label' => $label, 'abbrev' => $abbrev);
         }
 
         /* Add the INBOX on top of list if not in the filter list. */
@@ -192,7 +143,25 @@ class IMP_Folder {
      */
     function flist_IMP($filter = array(), $sub = null)
     {
-        return $this->flist(is_null($sub) ? $GLOBALS['prefs']->getValue('subscribe') : $sub, $filter);
+        return $this->flist(($sub === null) ? $GLOBALS['prefs']->getValue('subscribe') : $sub, $filter);
+    }
+
+    /**
+     * Clears the flist folder cache.
+     *
+     * @since IMP 4.2
+     */
+    function clearFlistCache()
+    {
+        if (!empty($_SESSION['imp']['cache']['folder_cache'])) {
+            require_once 'Horde/SessionObjects.php';
+            $sessionOb = &Horde_SessionObjects::singleton();
+            foreach ($_SESSION['imp']['cache']['folder_cache'] as $val) {
+                $sessionOb->setPruneFlag($val, true);
+            }
+            $_SESSION['imp']['cache']['folder_cache'] = array();
+        }
+        $this->_listCache = array();
     }
 
     /**
@@ -200,12 +169,11 @@ class IMP_Folder {
      *
      * @param array $folder_array  An array of full utf encoded folder names
      *                             to be deleted.
-     * @param boolean $subscribe   A boolean describing whether or not to use
-     *                             folder subscriptions.
+     * @param boolean $force       Delete folders even if they are fixed.
      *
      * @return boolean  Whether or not the folders were successfully deleted.
      */
-    function delete($folder_array, $subscribe)
+    function delete($folder_array, $force = false)
     {
         global $conf, $notification;
 
@@ -213,27 +181,23 @@ class IMP_Folder {
         $return_value = true;
         $deleted = array();
 
-        if ($subscribe) {
-            $sub_folders = $this->_listFolders(true);
-        }
+        $imp_imap = &IMP_IMAP::singleton();
 
         foreach ($folder_array as $folder) {
-            if (!imap_deletemailbox($_SESSION['imp']['stream'], $server . $folder)) {
+            if (!$force &&
+                !empty($conf['server']['fixed_folders']) &&
+                in_array(IMP::folderPref($folder, false), $conf['server']['fixed_folders'])) {
+                $notification->push(sprintf(_("The folder \"%s\" may not be deleted."), IMP::displayFolder($folder)), 'horde.error');
+                continue;
+            }
+            if (!imap_deletemailbox($imp_imap->stream(), $server . $folder)) {
                 $notification->push(sprintf(_("The folder \"%s\" was not deleted. This is what the server said"), IMP::displayFolder($folder)) .
                                     ': ' . imap_last_error(), 'horde.error');
                 $return_value = false;
             } else {
-                if ($subscribe &&
-                    in_array($server . $folder, $sub_folders) &&
-                    !imap_unsubscribe($_SESSION['imp']['stream'], $server . $folder)) {
-                    $notification->push(sprintf(_("The folder \"%s\" was deleted but you were not unsubscribed from it."), IMP::displayFolder($folder)), 'horde.warning');
-                    $return_value = false;
-                } else {
-                    $notification->push(sprintf(_("The folder \"%s\" was successfully deleted."), IMP::displayFolder($folder)), 'horde.success');
-                }
-
+                imap_unsubscribe($imp_imap->stream(), $server . $folder);
+                $notification->push(sprintf(_("The folder \"%s\" was successfully deleted."), IMP::displayFolder($folder)), 'horde.success');
                 $deleted[] = $folder;
-                unset($this->_existsResults[$folder]);
             }
         }
 
@@ -241,22 +205,47 @@ class IMP_Folder {
             /* Update the IMAP_Tree cache. */
             require_once IMP_BASE . '/lib/IMAP/Tree.php';
             $imaptree = &IMP_Tree::singleton();
-            if ($imaptree) {
-                $imaptree->delete($deleted);
-            }
+            $imaptree->delete($deleted);
 
-            /* Reset the folder cache. */
-            if ($conf['server']['cache_folders']) {
-                unset($_SESSION['imp']['cache']['folder_cache']);
-            }
-
-            /* Recreate Virtual Folders. */
-            $GLOBALS['imp_search']->sessionSetup();
+            $this->_onDelete($deleted);
         }
 
         return $return_value;
     }
 
+    /**
+     * Do the necessary cleanup/cache updates when deleting folders.
+     *
+     * @access private
+     *
+     * @param array $deleted  The list of deleted folders.
+     */
+    function _onDelete($deleted)
+    {
+        /* Update the mailbox cache. */
+        require_once IMP_BASE . '/lib/IMAP/MessageCache.php';
+        $msg_cache = &IMP_MessageCache::singleton();
+        $msg_cache->deleteMboxes($deleted);
+
+        /* Update the IMAP caches. */
+        require_once IMP_BASE . '/lib/IMAP/Cache.php';
+        $imap_cache = &IMP_IMAP_Cache::singleton();
+        foreach ($deleted as $val) {
+            $imap_cache->expireCache($val, 1 | 2 | 4);
+        }
+
+        /* Reset the folder cache. */
+        unset($_SESSION['imp']['cache']['folder_cache']);
+
+        /* Recreate Virtual Folders. */
+        $GLOBALS['imp_search']->sessionSetup(true);
+
+        /* Clear the folder from the sort prefs. */
+        foreach ($deleted as $val) {
+            IMP::setSort(null, null, $val, true);
+        }
+    }
+
     /**
      * Create a new IMAP folder if it does not already exist, and subcribe to
      * it as well if requested.
@@ -288,15 +277,20 @@ class IMP_Folder {
             return false;
         }
 
+        $display_folder = IMP::displayFolder($folder);
+
         /* Make sure we are not trying to create a duplicate folder */
         if ($this->exists($folder)) {
-            $notification->push(sprintf(_("The folder \"%s\" already exists"), IMP::displayFolder($folder)), 'horde.warning');
+            $notification->push(sprintf(_("The folder \"%s\" already exists"), $display_folder), 'horde.warning');
             return false;
         }
 
+        $imp_imap = &IMP_IMAP::singleton();
+        $server_folder = IMP::serverString($folder);
+
         /* Attempt to create the mailbox */
-        if (!imap_createmailbox($_SESSION['imp']['stream'], IMP::serverString($folder))) {
-            $notification->push(sprintf(_("The folder \"%s\" was not created. This is what the server said"), IMP::displayFolder($folder)) .
+        if (!imap_createmailbox($imp_imap->stream(), $server_folder)) {
+            $notification->push(sprintf(_("The folder \"%s\" was not created. This is what the server said"), $display_folder) .
                             ': ' . imap_last_error(), 'horde.error');
             return false;
         }
@@ -306,24 +300,22 @@ class IMP_Folder {
             unset($_SESSION['imp']['cache']['folder_cache']);
         }
 
-        /* If the user uses SUBSCRIBE, then add to the subscribe list */
-        $res = imap_subscribe($_SESSION['imp']['stream'], IMP::serverString($folder));
+        /* Subscribe to the folder. */
+        $res = imap_subscribe($imp_imap->stream(), $server_folder);
         if ($subscribe && !$res) {
-            $notification->push(sprintf(_("The folder \"%s\" was created but you were not subscribed to it."), IMP::displayFolder($folder)), 'horde.warning');
+            $notification->push(sprintf(_("The folder \"%s\" was created but you were not subscribed to it."), $display_folder), 'horde.warning');
         } else {
             /* The folder creation has been successful */
-            $notification->push(sprintf(_("The folder \"%s\" was successfully created."), IMP::displayFolder($folder)), 'horde.success');
+            $notification->push(sprintf(_("The folder \"%s\" was successfully created."), $display_folder), 'horde.success');
         }
 
         /* Update the IMAP_Tree object. */
         require_once IMP_BASE . '/lib/IMAP/Tree.php';
         $imaptree = &IMP_Tree::singleton();
-        if ($imaptree) {
-            $imaptree->insert($folder);
-        }
+        $imaptree->insert($folder);
 
         /* Recreate Virtual Folders. */
-        $GLOBALS['imp_search']->sessionSetup();
+        $GLOBALS['imp_search']->sessionSetup(true);
 
         return true;
     }
@@ -337,124 +329,78 @@ class IMP_Folder {
      */
     function exists($folder)
     {
-        /* Try the IMAP_Tree object first. */
         require_once IMP_BASE . '/lib/IMAP/Tree.php';
         $imaptree = &IMP_Tree::singleton();
-        if ($imaptree) {
-            $elt = $imaptree->get($folder);
-            if ($elt && !$imaptree->isContainer($elt)) {
-                return true;
-            }
+        $elt = $imaptree->get($folder);
+        if ($elt && !$imaptree->isContainer($elt)) {
+            return true;
         }
 
-        if (!isset($this->_existsResults[$folder])) {
-            $res = @imap_list($_SESSION['imp']['stream'], IMP::serverString(), $folder);
-            $this->_existsResults[$folder] = is_array($res);
-        }
-
-        return $this->_existsResults[$folder];
+        $imp_imap = &IMP_IMAP::singleton();
+        $ret = @imap_getmailboxes($imp_imap->stream(), IMP::serverString(), $folder);
+        return !empty($ret);
     }
 
     /**
      * Renames an IMAP folder. The subscription status remains the same.  All
      * subfolders will also be renamed.
      *
-     * @param string $old  The old utf encoded folder name.
-     * @param string $new  The new utf encoded folder name.
+     * @param string $old     The old utf encoded folder name.
+     * @param string $new     The new utf encoded folder name.
+     * @param boolean $force  Rename folders even if they are fixed.
      *
      * @return boolean  Whether or not all folder(s) were successfully renamed.
      */
-    function rename($old, $new)
+    function rename($old, $new, $force = false)
     {
-        global $conf, $notification;
-
         /* Don't try to rename from or to an empty string. */
         if (strlen($old) == 0 || strlen($new) == 0) {
             return false;
         }
-
-        $namespace_info = IMP::getNamespace($old);
-        $server = IMP::serverString();
-        $success = true;
-        $deleted = $inserted = array();
-
-        /* Get list of any folders that are underneath this one. */
-        $all_folders = array($server . $old);
-        $folder_list = imap_list($_SESSION['imp']['stream'], $server, $old . $namespace_info['delimiter'] . '*');
-        if (is_array($folder_list)) {
-            $all_folders = array_merge($folder_list, $all_folders);
-
-            /* Sort the folders in reverse order because some IMAP servers
-             * will automatically rename all folders when the base folder is
-             * renamed which will result in error messages if we rename from
-             * the bottom up. */
-            require_once IMP_BASE . '/lib/IMAP/Sort.php';
-            $ns_new = IMP::getNamespace($new);
-            $imap_sort = &new IMP_IMAP_Sort($ns_new['delimiter']);
-            $imap_sort->sortMailboxes($all_folders);
-            $all_folders = array_reverse($all_folders);
+        if (!$force &&
+            !empty($GLOBALS['conf']['server']['fixed_folders']) &&
+            in_array(IMP::folderPref($old, false), $GLOBALS['conf']['server']['fixed_folders'])) {
+            $GLOBALS['notification']->push(sprintf(_("The folder \"%s\" may not be renamed."), IMP::displayFolder($old)), 'horde.error');
+            return false;
         }
 
-        $sub_folders = $this->_listFolders(true);
+        $server = IMP::serverString();
+        $deleted = array($old);
+        $inserted = array($new);
+
+        require_once IMP_BASE . '/lib/IMAP/Tree.php';
+        $imaptree = &IMP_Tree::singleton();
+        $imp_imap = &IMP_IMAP::singleton();
+
+        /* Get list of any folders that are underneath this one. */
+        $all_folders = array_merge(array($old), $imaptree->folderList(IMPTREE_FLIST_UNSUB, $old));
+        $sub_folders = $imaptree->folderList();
+
+        if (!imap_renamemailbox($imp_imap->stream(), $server . $old, $server . $new)) {
+            $GLOBALS['notification']->push(sprintf(_("Renaming \"%s\" to \"%s\" failed. This is what the server said"), IMP::displayFolder($old), IMP::displayFolder($new)) . ': ' . imap_last_error(), 'horde.error');
+            return false;
+        }
+        $GLOBALS['notification']->push(sprintf(_("The folder \"%s\" was successfully renamed to \"%s\"."), IMP::displayFolder($old), IMP::displayFolder($new)), 'horde.success');
 
         foreach ($all_folders as $folder_old) {
-            $subscribe = false;
-
-            $old_pos = strpos($folder_old, '}') + 1;
+            $deleted[] = $folder_old;
 
             /* Get the new folder name. */
-            $folder_new = preg_replace('/' . preg_quote($old, '/') . '/', $new, $folder_old, 1);
+            $inserted[] = $folder_new = substr_replace($folder_old, $new, 0, strlen($old));
 
-            /* Get the folder names without the server prefix. */
-            $name_old = substr($folder_old, $old_pos);
-            $name_new = substr($folder_new, strpos($folder_new, '}') + 1);
-
-            /* Unsubscribe from current folder. */
+            /* Correctly set subscriptions on renamed folder. */
             if (in_array($folder_old, $sub_folders)) {
-                $subscribe = true;
-                imap_unsubscribe($_SESSION['imp']['stream'], $folder_old);
-            }
-
-            if (imap_renamemailbox($_SESSION['imp']['stream'], $folder_old, $folder_new)) {
-                if ($subscribe) {
-                    imap_subscribe($_SESSION['imp']['stream'], $folder_new);
-                }
-
-                $deleted[] = $name_old;
-                $inserted[] = $name_new;
-
-                $notification->push(sprintf(_("The folder \"%s\" was successfully renamed to \"%s\"."), IMP::displayFolder($name_old), IMP::displayFolder($name_new)), 'horde.success');
-
-                unset($this->_existsResults[$name_old]);
-
-                // Change current mailbox if current mailbox was renamed.
-                if ($_SESSION['imp']['mailbox'] == $name_old) {
-                    $_SESSION['imp']['mailbox'] = $name_new;
-                }
-            } else {
-                $notification->push(sprintf(_("Renaming \"%s\" to \"%s\" failed. This is what the server said"), IMP::displayFolder($name_old), IMP::displayFolder($name_new)) . ': ' . imap_last_error(), 'horde.error');
-                $success = false;
+                imap_unsubscribe($imp_imap->stream(), $server . $folder_old);
+                imap_subscribe($imp_imap->stream(), $server . $folder_new);
             }
         }
 
         if (!empty($deleted)) {
-            /* Update the IMP_Tree cache. */
-            require_once IMP_BASE . '/lib/IMAP/Tree.php';
-            $imaptree = &IMP_Tree::singleton();
-            if ($imaptree) {
-                $imaptree->rename($deleted, $inserted);
-            }
-
-            /* Reset the folder cache. */
-            if ($conf['server']['cache_folders']) {
-                unset($_SESSION['imp']['cache']['folder_cache']);
-            }
-
-            /* Recreate Virtual Folders. */
-            $GLOBALS['imp_search']->sessionSetup();
+            $imaptree->rename($deleted, $inserted);
+            $this->_onDelete($deleted);
         }
 
-        return $success;
+        return true;
     }
 
     /**
@@ -468,7 +414,7 @@ class IMP_Folder {
      */
     function subscribe($folder_array)
     {
-        global $conf, $notification;
+        global $notification;
 
         $return_value = true;
         $subscribed = array();
@@ -478,9 +424,11 @@ class IMP_Folder {
             return false;
         }
 
+        $imp_imap = &IMP_IMAP::singleton();
+
         foreach ($folder_array as $folder) {
             if ($folder != ' ') {
-                if (!imap_subscribe($_SESSION['imp']['stream'], IMP::serverString($folder))) {
+                if (!imap_subscribe($imp_imap->stream(), IMP::serverString($folder))) {
                     $notification->push(sprintf(_("You were not subscribed to \"%s\". Here is what the server said"), IMP::displayFolder($folder)) . ': ' . imap_last_error(), 'horde.error');
                     $return_value = false;
                 } else {
@@ -494,14 +442,10 @@ class IMP_Folder {
             /* Initialize the IMAP_Tree object. */
             require_once IMP_BASE . '/lib/IMAP/Tree.php';
             $imaptree = &IMP_Tree::singleton();
-            if ($imaptree) {
-                $imaptree->subscribe($subscribed);
-            }
+            $imaptree->subscribe($subscribed);
 
             /* Reset the folder cache. */
-            if ($conf['server']['cache_folders']) {
-                unset($_SESSION['imp']['cache']['folder_cache']);
-            }
+            unset($_SESSION['imp']['cache']['folder_cache']);
         }
 
         return $return_value;
@@ -518,7 +462,7 @@ class IMP_Folder {
      */
     function unsubscribe($folder_array)
     {
-        global $conf, $notification;
+        global $notification;
 
         $return_value = true;
         $unsubscribed = array();
@@ -528,11 +472,13 @@ class IMP_Folder {
             return false;
         }
 
+        $imp_imap = &IMP_IMAP::singleton();
+
         foreach ($folder_array as $folder) {
             if ($folder != ' ') {
                 if (strcasecmp($folder, 'INBOX') == 0) {
                     $notification->push(sprintf(_("You cannot unsubscribe from \"%s\"."), IMP::displayFolder($folder)), 'horde.error');
-                } elseif (!imap_unsubscribe($_SESSION['imp']['stream'], IMP::serverString($folder))) {
+                } elseif (!imap_unsubscribe($imp_imap->stream(), IMP::serverString($folder))) {
                     $notification->push(sprintf(_("You were not unsubscribed from \"%s\". Here is what the server said"), IMP::displayFolder($folder)) . ': ' . imap_last_error(), 'horde.error');
                     $return_value = false;
                 } else {
@@ -546,14 +492,10 @@ class IMP_Folder {
             /* Initialize the IMAP_Tree object. */
             require_once IMP_BASE . '/lib/IMAP/Tree.php';
             $imaptree = &IMP_Tree::singleton();
-            if ($imaptree) {
-                $imaptree->unsubscribe($unsubscribed);
-            }
+            $imaptree->unsubscribe($unsubscribed);
 
             /* Reset the folder cache. */
-            if ($conf['server']['cache_folders']) {
-                unset($_SESSION['imp']['cache']['folder_cache']);
-            }
+            unset($_SESSION['imp']['cache']['folder_cache']);
         }
 
         return $return_value;
@@ -578,13 +520,13 @@ class IMP_Folder {
         $body = '';
 
         if (is_array($folder_list)) {
-            require_once IMP_BASE . '/lib/IMAP.php';
             $imp_imap = &IMP_IMAP::singleton();
+            $stream = $imp_imap->stream();
             foreach ($folder_list as $folder) {
-                $imp_imap->changeMbox($folder, OP_READONLY);
-                $count = imap_num_msg($_SESSION['imp']['stream']);
+                $imp_imap->changeMbox($folder, IMP_IMAP_AUTO);
+                $count = imap_num_msg($stream);
                 for ($i = 1; $i <= $count; $i++) {
-                    $h = imap_header($_SESSION['imp']['stream'], $i);
+                    $h = imap_header($stream, $i);
                     $from = '<>';
                     if (isset($h->from[0])) {
                         if (isset($h->from[0]->mailbox) && isset($h->from[0]->host)) {
@@ -596,8 +538,8 @@ class IMP_Folder {
                        require a space in front of single digit days. */
                     $date = sprintf('%s %2s %s', date('D M', $h->udate), date('j', $h->udate), date('H:i:s Y', $h->udate));
                     $body .= 'From ' . $from . ' ' . $date . "\n";
-                    $body .= str_replace("\r\n", "\n", imap_fetchheader($_SESSION['imp']['stream'], $i, FT_PREFETCHTEXT));
-                    $body .= str_replace("\r\n", "\n", imap_body($_SESSION['imp']['stream'], $i, FT_PEEK) . "\n");
+                    $body .= str_replace("\r\n", "\n", imap_fetchheader($stream, $i, FT_PREFETCHTEXT));
+                    $body .= str_replace("\r\n", "\n", imap_body($stream, $i, FT_PEEK)) . "\n";
                 }
             }
         }
@@ -616,10 +558,12 @@ class IMP_Folder {
      */
     function importMbox($folder, $mbox)
     {
-        $target = IMP::ServerString() . $folder;
-
         $message = '';
         $msgcount = 0;
+        $target = IMP::serverString($folder);
+
+        $imp_imap = &IMP_IMAP::singleton();
+        $stream = $imp_imap->stream();
 
         $fd = fopen($mbox, "r");
         while (!feof($fd)) {
@@ -627,11 +571,7 @@ class IMP_Folder {
 
             if (preg_match('/From (.+@.+|- )/A', $line)) {
                 if (!empty($message)) {
-                    // Make absolutely sure there are no bare newlines.
-                    $message = preg_replace("|([^\r])\n|", "\\1\r\n", $message);
-                    $message = str_replace("\n\n", "\n\r\n", $message);
-
-                    if (imap_append($_SESSION['imp']['stream'], $target, $message)) {
+                    if (@imap_append($stream, $target, IMP::removeBareNewlines($message))) {
                         $msgcount++;
                     }
                 }
@@ -643,11 +583,7 @@ class IMP_Folder {
         fclose($fd);
 
         if (!empty($message)) {
-            // Make absolutely sure there are no bare newlines.
-            $message = preg_replace("|([^\r])\n|", "\\1\r\n", $message);
-            $message = str_replace("\n\n", "\n\r\n", $message);
-
-            if (imap_append($_SESSION['imp']['stream'], $target, $message)) {
+            if (@imap_append($stream, $target, IMP::removeBareNewlines($message))) {
                 $msgcount++;
             }
         }
@@ -655,34 +591,4 @@ class IMP_Folder {
         return ($msgcount > 0) ? $msgcount : false;
     }
 
-    /**
-     * Get a list of all folders on the server (including any folders in
-     * hidden namespaces).
-     *
-     * @access private
-     *
-     * @param boolean $sub  List only subscribed folders?
-     *
-     * @return array  An array of folders names.
-     */
-    function _listFolders($sub = false)
-    {
-        $list = array();
-        $listcmd = ($sub) ? 'imap_lsub' : 'imap_list';
-        $server = IMP::serverString();
-
-        /* According to RFC 3501 [6.3.8], the '*' wildcard doesn't
-         * necessarily match all visible mailboxes.  So we have to go
-         * through each namespace separately, even though we may duplicate
-         * mailboxes. */
-        foreach ($_SESSION['imp']['namespace'] as $val) {
-            $hiddenboxes = $listcmd($_SESSION['imp']['stream'], $server, $val['name'] . '*');
-            if (is_array($hiddenboxes)) {
-                $list = array_unique(array_merge($list, $hiddenboxes));
-            }
-        }
-
-        return array_values($list);
-    }
-
 }
