@@ -10,9 +10,9 @@ define('IMP_IMAPCLIENT_CONTINUATION', 2);
  *
  * TODO: This should eventually be moved to Horde 4.0/framework.
  *
- * $Horde: imp/lib/IMAP/Client.php,v 1.21.2.29 2006/06/07 18:07:05 slusarz Exp $
+ * $Horde: imp/lib/IMAP/Client.php,v 1.21.2.34 2008/01/02 11:31:29 jan Exp $
  *
- * Copyright 2005-2006 Michael Slusarz <slusarz@horde.org>
+ * Copyright 2005-2008 The Horde Project (http://www.horde.org/)
  *
  * Based on code from:
  *   + auth.php (1.49)
@@ -351,7 +351,7 @@ class IMP_IMAPClient {
         /* Use md5 authentication, if available. But no need to use special
          * authentication if we are already using an encrypted connection. */
         $auth_methods = $this->queryCapability('AUTH');
-        if ((!$this->_usessl || !$this->_usetls) && !empty($auth_methods)) {
+        if (!$this->_usessl && !$this->_usetls && !empty($auth_methods)) {
             if (in_array('CRAM-MD5', $auth_methods)) {
                 $imap_auth_mech[] = 'cram-md5';
             }
@@ -504,7 +504,7 @@ class IMP_IMAPClient {
      */
     function _capability()
     {
-        if (!is_null($this->_capability)) {
+        if ($this->_capability !== null) {
             return;
         }
 
@@ -533,7 +533,8 @@ class IMP_IMAPClient {
     /**
      * Returns whether the IMAP server supports the given capability.
      *
-     * @param string $capability  The capability string to query.
+     * @param string $capability  The capability string to query. If null,
+     *                            returns the entire capability array.
      *
      * @param mixed  True if the server supports the queried capability,
      *               false if it doesn't, or an array if the capability can
@@ -542,7 +543,7 @@ class IMP_IMAPClient {
     function queryCapability($capability)
     {
         $this->_capability();
-        return isset($this->_capability[$capability]) ? $this->_capability[$capability] : false;
+        return ($capability === null) ? $this->_capability : (isset($this->_capability[$capability]) ? $this->_capability[$capability] : false);
     }
 
     /**
@@ -561,7 +562,7 @@ class IMP_IMAPClient {
      *   (
      *     [name] => (string)
      *     [delimiter] => (string)
-     *     [type] => [personal|others|shared] (string)
+     *     [type] => [personal|other|shared] (string)
      *     [hidden] => (boolean)
      *   )
      *
@@ -573,21 +574,20 @@ class IMP_IMAPClient {
      * </pre>
      *                Returns PEAR_Error object on error.
      */
-    function namespace($additional = array())
+    function getNamespace($additional = array())
     {
-        if (!is_null($this->_namespace)) {
+        if ($this->_namespace !== null) {
             return $this->_namespace;
         }
 
         $namespace_array = array(
             1 => 'personal',
-            2 => 'others',
+            2 => 'other',
             3 => 'shared'
         );
 
         if ($this->queryCapability('NAMESPACE')) {
-            /*
-             * According to rfc2342 response from NAMESPACE command is:
+            /* According to RFC 2342, response from NAMESPACE command is:
              * * NAMESPACE (PERSONAL NAMESPACES) (OTHER_USERS NAMESPACE) (SHARED NAMESPACES)
              */
             $read = $this->_runCommand('NAMESPACE');
@@ -629,7 +629,9 @@ class IMP_IMAPClient {
                     $this->_namespace[$val] = array('name' => $val, 'delimiter' => $data[1], 'type' => $namespace_array[3], 'hidden' => true);
                 }
             }
-        } else {
+        }
+
+        if (empty($this->_namespace)) {
             $res = $this->_runCommand('LIST "" ""');
             if (is_a($res, 'PEAR_Error')) {
                 Horde::logMessage($res, __FILE__, __LINE__, PEAR_LOG_ERR);
