@@ -1,24 +1,33 @@
 <?php
+/**
+ * @package Horde_MIME
+ */
 
 require_once 'Horde/MIME/Contents.php';
 // TODO: for BC - Remove for Horde 4.0
-require IMP_BASE . '/config/mime_drivers.php';
+if (is_callable(array('Horde', 'loadConfiguration'))) {
+    $result = Horde::loadConfiguration('mime_drivers.php', array('mime_drivers', 'mime_drivers_map'), 'imp');
+    if (!is_a($result, 'PEAR_Error')) {
+        extract($result);
+    }
+} else {
+    require IMP_BASE . '/config/mime_drivers.php';
+}
 
 /**
  * The IMP_Contents:: class extends the MIME_Contents:: class and contains
  * all functions related to handling the content and output of mail messages
  * in IMP.
  *
- * $Horde: imp/lib/MIME/Contents.php,v 1.153.4.44 2007/01/02 13:54:59 jan Exp $
+ * $Horde: imp/lib/MIME/Contents.php,v 1.153.4.60 2008/08/10 09:38:14 jan Exp $
  *
- * Copyright 2002-2007 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
  *
- * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
- * @since   IMP 4.0
- * @package IMP
+ * @author  Michael Slusarz <slusarz@horde.org>
+ * @package Horde_MIME
  */
 class IMP_Contents extends MIME_Contents {
 
@@ -32,6 +41,8 @@ class IMP_Contents extends MIME_Contents {
     /**
      * The MIME part id of the message body.
      *
+     * @todo Copied to MIME_Contents but kept here for BC.
+     *
      * @var integer
      */
     var $_body_id;
@@ -64,6 +75,15 @@ class IMP_Contents extends MIME_Contents {
      */
     var $_strip = false;
 
+    /**
+     * List of all downloadable parts.
+     *
+     * @todo Copied to MIME_Contents but kept here for BC.
+     *
+     * @var array
+     */
+    var $_downloads = null;
+
     /**
      * Attempts to return a reference to a concrete IMP_Contents instance.
      * If an IMP_Contents object is currently stored in the local cache,
@@ -72,38 +92,44 @@ class IMP_Contents extends MIME_Contents {
      * available at any one time.
      *
      * This method must be invoked as:
-     *   $imp_contents = &IMP_Contents::singleton($index);
+     *   $imp_contents = &IMP_Contents::singleton($in);
      *
-     * @param string $index  The IMAP message mailbox/index. The index should
-     *                       be in the following format:
-     * <pre>
-     * msg_id IMP_IDX_SEP msg_folder
-     *   msg_id      = Message index of the message
-     *   IMP_IDX_SEP = IMP constant used to separate index/folder
-     *   msg_folder  = The full folder name containing the message index
-     * </pre>
+     * @param mixed $in  Either an index string (see IMP_Contents::singleton()
+     *                   for the format) or a MIME_Message object.
      *
      * @return IMP_Contents  The IMP_Contents object or null.
      */
-    function &singleton($index)
+    function &singleton($in)
     {
         static $instance = array();
 
-        $signature = IMP_Contents::_createCacheID($index);
-        if (isset($instance[$signature])) {
-            return $instance[$signature];
+        $sig = IMP_Contents::_createCacheID($in);
+        if (isset($instance[$sig])) {
+            return $instance[$sig];
         }
 
-        $instance[$signature] = &IMP_Contents::getCache($signature);
-        if (empty($instance[$signature])) {
-            $instance[$signature] = &new IMP_Contents($index);
+        $instance[$sig] = &IMP_Contents::getCache($sig);
+
+        // The returned instance might be a MIMP_Contents object.  If not in
+        // MIMP context, then just create a new object instead.
+        if (!empty($instance[$sig]) &&
+            !is_subclass_of($instance[$sig], 'MIME_Contents')) {
+            unset($instance[$sig]);
         }
 
-        return $instance[$signature];
+        if (empty($instance[$sig])) {
+            $instance[$sig] = new IMP_Contents($in);
+            $message = $instance[$sig]->getMIMEMessage();
+            if (is_a($message, 'PEAR_Error')) {
+                return $message;
+            }
+        }
+
+        return $instance[$sig];
     }
 
     /**
-     * Constructor
+     * Constructor.
      *
      * @param mixed $in  Either an index string (see IMP_Contents::singleton()
      *                   for the format) or a MIME_Message object.
@@ -112,19 +138,19 @@ class IMP_Contents extends MIME_Contents {
     {
         if (is_a($in, 'MIME_Message')) {
             $ob = $in;
+            $in = null;
         } else {
             list($this->_index, $this->_mailbox) = explode(IMP_IDX_SEP, $in);
 
             /* Get the MIME_Structure object for the given index. */
-            require_once IMP_BASE . '/lib/IMAP.php';
-            $imp_imap = &IMP_IMAP::singleton();
-            $imp_imap->changeMbox($this->_mailbox);
-            $structure = @imap_fetchstructure($_SESSION['imp']['stream'], $this->_index, FT_UID);
-            if (!is_object($structure)) {
+            require_once IMP_BASE . '/lib/IMAP/MessageCache.php';
+            $msg_cache = &IMP_MessageCache::singleton();
+            $ret = $msg_cache->retrieve($this->_mailbox, array($this->_index), 8);
+            if (!isset($ret[$this->_index]) || !isset($ret[$this->_index]->structure)) {
+                $this->_message = PEAR::raiseError(_("Error displaying message."));
                 return;
             }
-            require_once 'Horde/MIME/Structure.php';
-            $ob = &MIME_Structure::parse($structure);
+            $ob = &Util::cloneObject($ret[$this->_index]->structure);
         }
 
         switch ($GLOBALS['prefs']->getValue('attachment_display')) {
@@ -152,8 +178,9 @@ class IMP_Contents extends MIME_Contents {
     function getBody()
     {
         if (empty($this->_body)) {
-            $this->_body = @imap_body($_SESSION['imp']['stream'], $this->_index, FT_UID | FT_PEEK);
-            $this->_body = str_replace("\r\n", "\n", $this->_body);
+            $imp_imap = &IMP_IMAP::singleton();
+            $imp_imap->changeMbox($this->_mailbox, IMP_IMAP_AUTO);
+            $this->_body = @imap_body($imp_imap->stream(), $this->_index, FT_UID | FT_PEEK);
         }
 
         return $this->_body;
@@ -169,8 +196,9 @@ class IMP_Contents extends MIME_Contents {
     function getBodyPart($id)
     {
         if (!isset($this->_bodypart[$id])) {
-            $this->_bodypart[$id] = @imap_fetchbody($_SESSION['imp']['stream'], $this->_index, $id, FT_UID | FT_PEEK);
-            $this->_bodypart[$id] = str_replace("\r\n", "\n", $this->_bodypart[$id]);
+            $imp_imap = &IMP_IMAP::singleton();
+            $imp_imap->changeMbox($this->_mailbox, IMP_IMAP_AUTO);
+            $this->_bodypart[$id] = imap_fetchbody($imp_imap->stream(), $this->_index, $id, FT_UID | FT_PEEK);
         }
 
         return isset($this->_bodypart[$id]) ? $this->_bodypart[$id] : '';
@@ -179,11 +207,14 @@ class IMP_Contents extends MIME_Contents {
     /**
      * Allow attachments to be stripped by providing a link in summary view?
      *
-     * @param boolean $strip  Should the strip links be generated?
+     * @param boolean $strip         Should the strip links be generated?
+     * @param string $message_token  The message token to append to the strip
+     *                               URL.
      */
-    function setStripLink($strip = false)
+    function setStripLink($strip = false, $message_token = null)
     {
         $this->_strip = $strip;
+        $this->_message_token = $message_token;
     }
 
     /**
@@ -200,7 +231,7 @@ class IMP_Contents extends MIME_Contents {
      */
     function partSummary(&$mime_part, $guess = false)
     {
-        global $conf, $registry;
+        global $registry;
 
         $summary = parent::partSummary($mime_part, $guess);
 
@@ -226,9 +257,9 @@ class IMP_Contents extends MIME_Contents {
         if (($mime_part->getPrimaryType() == 'image') &&
             $registry->hasMethod('images/selectGalleries') &&
             ($image_app = $registry->hasMethod('images/saveImage'))) {
-            Horde::addScriptFile('popup.js');
-            $url = Util::addParameter(Horde::applicationUrl('saveimage.php'), array('index' => ($this->_index . IMP_IDX_SEP . $this->_mailbox), 'id' => $mime_part->getMIMEId()));
-            $summary[] = Horde::link('#', _("Save Image in Gallery"), null, null, "popup_imp('" . $url . "',450,200); return false;") . '<img src="' . $registry->get('icon', $image_app) . '" alt="' . _("Save Image in Gallery") . '" title="' . _("Save Image in Gallery") . '" /></a>';
+            Horde::addScriptFile('prototype.js', 'imp', true);
+            Horde::addScriptFile('popup.js', 'imp', true);
+            $summary[] = Horde::link('#', _("Save Image in Gallery"), null, null, IMP::popupIMPString('saveimage.php', array('index' => ($this->_index . IMP_IDX_SEP . $this->_mailbox), 'id' => $mime_part->getMIMEId()), 450, 200) . "return false;") . '<img src="' . $registry->get('icon', $image_app) . '" alt="' . _("Save Image in Gallery") . '" title="' . _("Save Image in Gallery") . '" /></a>';
         } else {
             $summary[] = null;
         }
@@ -238,7 +269,7 @@ class IMP_Contents extends MIME_Contents {
             !$mime_part->getInformation('rfc822_part')) {
             $url = Horde::selfUrl(true);
             $url = Util::removeParameter($url, array('actionID', 'imapid', 'index'));
-            $url = Util::addParameter($url, array('actionID' => 'strip_attachment', 'imapid' => $this->_getMIMEKey($mime_part, false), 'index' => $this->getMessageIndex()));
+            $url = Util::addParameter($url, array('actionID' => 'strip_attachment', 'imapid' => $this->_getMIMEKey($mime_part, false), 'index' => $this->_index, 'message_token' => $this->_message_token));
             $summary[] = Horde::link($url, _("Strip Attachment"), null, null, "return window.confirm('" . addslashes(_("Are you sure you wish to PERMANENTLY delete this attachment?")) . "');") . Horde::img('delete.png', _("Strip Attachment"), null, $registry->getImageDir('horde')) . '</a>';
         } else {
             $summary[] = null;
@@ -261,14 +292,8 @@ class IMP_Contents extends MIME_Contents {
      */
     function urlView(&$mime_part, $actionID, $params = array(), $dload = false)
     {
-        require_once IMP_BASE . '/lib/Search.php';
-
         /* Add the necessary local parameters. */
-        $params = array_merge($params, IMP::getSearchParameters($_SESSION['imp']['thismailbox']));
-        $params['index'] = $this->_index;
-        if (!isset($params['mailbox'])) {
-            $params['mailbox'] = $_SESSION['imp']['mailbox'];
-        }
+        $params = array_merge($params, IMP::getIMPMboxParameters($GLOBALS['imp_mbox']['mailbox'], $this->_index, $this->_mailbox));
 
         /* Should this be a download link? */
         $dload = (($actionID == 'download_attach') ||
@@ -301,7 +326,7 @@ class IMP_Contents extends MIME_Contents {
 
         /* If download attachment, add the 'thismailbox' param. */
         if ($actionID == 'download_attach') {
-            $params['viewparams']['thismailbox'] = $_SESSION['imp']['thismailbox'];
+            $params['viewparams']['thismailbox'] = $this->_mailbox;
         }
 
         if ($mime_part->getInformation('viewparams')) {
@@ -331,8 +356,10 @@ class IMP_Contents extends MIME_Contents {
      */
     function &getHeaderOb()
     {
-        require_once IMP_BASE . '/lib/MIME/Headers.php';
-        $imp_headers = &new IMP_Headers($this->_index);
+        require_once IMP_BASE . '/lib/IMAP/MessageCache.php';
+        $msg_cache = &IMP_MessageCache::singleton();
+        $ret = $msg_cache->retrieve($this->_mailbox, array($this->_index), 32);
+        $imp_headers = &Util::cloneObject($ret[$this->_index]->header);
         return $imp_headers;
     }
 
@@ -346,6 +373,18 @@ class IMP_Contents extends MIME_Contents {
         return $this->_index;
     }
 
+    /**
+     * Returns the IMAP mailbox for the current message.
+     *
+     * @since IMP 4.2.1
+     *
+     * @return string  The message mailbox.
+     */
+    function getMessageMailbox()
+    {
+        return $this->_mailbox;
+    }
+
     /**
      * Rebuild the MIME_Part structure of a message from IMAP data.
      * This will store IMAP data in all parts of the message - for example,
@@ -387,7 +426,8 @@ class IMP_Contents extends MIME_Contents {
 
         if ($part->getPrimaryType() == 'multipart') {
             /* Recursively process any subparts. */
-            foreach ($part->getParts() as $mime) {
+            $parts = $part->getParts();
+            while ($mime = array_shift($parts)) {
                 $this->_rebuildMessage($mime);
             }
         }
@@ -402,15 +442,23 @@ class IMP_Contents extends MIME_Contents {
      */
     function renderMIMEPart(&$mime_part)
     {
-        $this->_setContents($mime_part);
-        $text = parent::renderMIMEPart($mime_part);
+        $viewer = $this->getMIMEViewer($mime_part);
+        if (!is_a($viewer, 'PEAR_Error') &&
+            $viewer->getConfigParam('limit_inline_size') &&
+            ($mime_part->getBytes() > $viewer->getConfigParam('limit_inline_size'))) {
+            $text = $this->formatStatusMsg(array(_("This message part cannot be viewed because it is too large."), sprintf(_("Click %s to download the data."), $this->linkView($mime_part, 'download_attach', _("HERE")))));
+        } else {
+            $this->_setContents($mime_part);
+            $text = parent::renderMIMEPart($mime_part);
 
-        /* Convert textual emoticons into graphical ones - but only for
-         * text parts. */
-        if (($mime_part->getPrimaryType() == 'text') &&
-            $GLOBALS['prefs']->getValue('emoticons')) {
-            require_once 'Horde/Text/Filter.php';
-            $text = Text_Filter::filter($text, 'emoticons', array('entities' => true));
+            /* Convert textual emoticons into graphical ones - but only for
+             * text parts. */
+            if (($mime_part->getPrimaryType() == 'text') &&
+                !$this->viewAsAttachment() &&
+                $GLOBALS['prefs']->getValue('emoticons')) {
+                require_once 'Horde/Text/Filter.php';
+                $text = Text_Filter::filter($text, 'emoticons', array('entities' => true));
+            }
         }
 
         return $text;
@@ -423,30 +471,8 @@ class IMP_Contents extends MIME_Contents {
      */
     function _addCacheShutdown()
     {
-        /* Don't cache the $_bodypart array since we most likely will not use
-         * them in the cached object (since the cache will generally be used
-         * to view parts that weren't displayed originally).
-         * However, we will lose the benefit of not having to
-         * make calls to the mail server on subsequent page loads, so allow
-         * users to change this behavior.  Only cache small messages as
-         * storing large messages will make the session size too large and
-         * will tax storage and performance on the server. */
-        if (empty($GLOBALS['conf']['server']['cache_msgbody'])) {
-            $this->_bodypart = array();
-        } else {
-            $size = 0;
-            foreach ($this->_bodypart as $key => $val) {
-                $part_size = strlen($val);
-                if (($size + $part_size) < 10240) {
-                    $size += $part_size;
-                } else {
-                    unset($this->_bodypart[$key]);
-                }
-            }
-        }
-
-        /* Never cache the full body part as we use this item much less
-         * often than specific message body parts. */
+        /* Don't cache bodypart data in the session. */
+        $this->_bodypart = array();
         $this->_body = null;
 
         parent::_addCacheShutdown();
@@ -459,65 +485,54 @@ class IMP_Contents extends MIME_Contents {
      */
     function getFromAddress()
     {
-        require_once IMP_BASE . '/lib/MIME/Headers.php';
-        $headers = &new IMP_Headers($this->getMessageIndex());
+        $headers = &$this->getHeaderOb();
         return $headers->getFromAddress();
     }
 
     /**
      * Generate the list of MIME IDs to use for download all.
      *
-     * @param boolean $forward  Generate a list of items that can be forwarded,
-     *                          rather than downloaded.
+     * @todo  Copied to MIME_Contents but kept here for BC.
      *
      * @return array  The list of MIME IDs that should be downloaded when
      *                downloading all attachments.
      */
-    function getDownloadAllList($forward = false)
+    function getDownloadAllList()
     {
-        $downloads = array();
+        if (is_array($this->_downloads)) {
+            return $this->_downloads;
+        }
 
+        $this->_downloads = array();
         $bodyid = $this->findBody();
 
-        /* Here is what we consider 'forwardable':
-         * All parts > 0
-         * Part 0 if not multipart/mixed
-         * NOT the body part (if one exists)
-         *
-         * Here is what we consider 'downloadable':
+        /* Here is what we consider 'downloadable':
          * All parts not 'multipart/*' and 'message/*' except for
          *  'message/rfc822'
          * All parts that are not PGP or S/MIME signature information
-         * NOT the body part (if one exists) */
+         * NOT the body part (if one exists)
+         * Parts that are either marked a 'attachment' or have a filename. */
         foreach ($this->_message->contentTypeMap() as $key => $val) {
-            if ($key == $bodyid) {
+            if ($key === $bodyid) {
                 continue;
             }
 
-            if ($forward) {
-                if ($key == 0) {
-                    if ($val != 'multipart/mixed') {
-                        $downloads[] = $key;
-                        break;
-                    }
-                } elseif (strpos($key, '.') === false) {
-                    $downloads[] = $key;
-                }
-            } else {
-                if (strpos($val, 'message/') === 0) {
-                    if (strpos($val, 'message/rfc822') === 0) {
-                        $downloads[] = $key;
-                    }
-                } elseif (((intval($key) != 1) && (strpos($val, '1.') === false)) &&
-                          ((strpos($val, 'multipart/') === false)) &&
-                          (strpos($val, 'application/x-pkcs7-signature') === false) &&
-                          (strpos($val, 'application/pkcs7-signature') === false)) {
-                    $downloads[] = $key;
+            $mime_part = $this->getMIMEPart($key);
+
+            if (strpos($val, 'message/') === 0) {
+                if (strpos($val, 'message/rfc822') === 0) {
+                    $this->_downloads[] = $key;
                 }
+            } elseif ((($mime_part->getDisposition() == 'attachment') ||
+                       $mime_part->getContentTypeParameter('name')) &&
+                      (strpos($val, 'multipart/') === false) &&
+                      (strpos($val, 'application/x-pkcs7-signature') === false) &&
+                      (strpos($val, 'application/pkcs7-signature') === false)) {
+                $this->_downloads[] = $key;
             }
         }
 
-        return $downloads;
+        return $this->_downloads;
     }
 
     /**
@@ -541,6 +556,101 @@ class IMP_Contents extends MIME_Contents {
         return $url;
     }
 
+    /**
+     * Processes a MIME_Part and stores the display information in the internal
+     * class variables.
+     *
+     * @param MIME_Part &$mime_part  The MIME_Part object to process.
+     *
+     * @return string  The rendered text.
+     */
+    function buildMessagePart(&$mime_part)
+    {
+        $key = $this->_getMIMEKey($mime_part);
+        $attachmentList = in_array($key, $this->getDownloadAllList());
+
+        /* If we can't display the part inline, add it to the attachment
+           list. If the MIME ID of the current part is '0', then force a
+           render of the part (since it is the base part and, without
+           attempting to render, the message will ALWAYS appear empty. */
+        if (!$this->canDisplayInline($mime_part) &&
+            ($mime_part->getMIMEId() != 0)) {
+            /* Not displaying inline; add to the attachments list. */
+            if ($attachmentList) {
+                if (($this->_displayType == MIME_CONTENTS_DISPLAY_TYPE_LIST) ||
+                    ($this->_displayType == MIME_CONTENTS_DISPLAY_TYPE_BOTH)) {
+                    $this->setSummary($mime_part, 'attachment');
+                }
+                if (($this->_displayType == MIME_CONTENTS_DISPLAY_TYPE_INLINE) ||
+                    ($this->_displayType == MIME_CONTENTS_DISPLAY_TYPE_BOTH)) {
+                    $this->setSummary($mime_part, 'part');
+                }
+            }
+
+            /* Check to see if any attachment information can be rendered by
+               the MIME_Viewer. */
+            $msg = $this->renderMIMEAttachmentInfo($mime_part);
+            if (!empty($msg)) {
+                $this->_parts[$key] = $msg;
+            }
+        } else {
+            $msg = $this->renderMIMEPart($mime_part);
+            if (!$this->_attach822) {
+                $this->_parts[$key] = $msg;
+            }
+
+            /* Some MIME_Viewers set the summary by themelves, so only
+             * add to attachment/inline lists if nothing has been set
+             * as of yet. */
+            $bodyid = $this->findBody();
+            if ((($mime_part->getType() != 'multipart/mixed') ||
+                 !empty($msg)) &&
+                ($key !== false) &&
+                !$this->getIndex($key, 'summary') &&
+                (($this->_displayType == MIME_CONTENTS_DISPLAY_TYPE_LIST) ||
+                 ($this->_displayType == MIME_CONTENTS_DISPLAY_TYPE_BOTH))) {
+
+                if ($attachmentList) {
+                    /* TODO: Remove this check for Horde 4.0. */
+                    if ($mime_part->getType() != 'multipart/alternative') {
+                        $this->setSummary($mime_part, 'attachment');
+                    }
+                } elseif (($bodyid !== null) && ($key != $bodyid)) {
+                    $this->setSummary($mime_part, 'part');
+                }
+            } elseif (($mime_part->getType() == 'multipart/mixed') &&
+                      empty($msg) &&
+                      !is_null($bodyid)) {
+                // See Bug 5157
+                $this->setSummary($this->_message->getPart($bodyid), 'part');
+            }
+        }
+
+        if ($mime_part->getInformation('header')) {
+            /* If this is message/rfc822 part, and it is marked as an
+             * attachment, we need to let future calls to buildMessagePart()
+             * know that it should mark embedded parts as not viewable
+             * inline. */
+            $increment_822 = false;
+            if (($mime_part->getType() == 'message/rfc822') &&
+                ($mime_part->getDisposition() == 'attachment')) {
+                $this->_attach822++;
+                $increment_822 = true;
+            }
+
+            $parts = $mime_part->getParts();
+            while ($part = array_shift($parts)) {
+                $msg .= $this->buildMessagePart($part);
+            }
+
+            if ($increment_822) {
+                $this->_attach822--;
+            }
+        }
+
+        return $msg;
+    }
+
     /**
      * Prints out the status message for a given MIME Part.
      *
@@ -563,11 +673,15 @@ class IMP_Contents extends MIME_Contents {
      * Finds the main "body" text part (if any) in a message.
      * "Body" data is the first text part in the base MIME part.
      *
+     * @todo  Copied to MIME_Contents but kept here for BC.
+     *
+     * @param string $subtype  Specifically search for this subtype.
+     *
      * @return string  The MIME ID of the main "body" part.
      */
-    function findBody()
+    function findBody($subtype = null)
     {
-        if (isset($this->_body_id)) {
+        if (isset($this->_body_id) && ($subtype === null)) {
             return $this->_body_id;
         }
 
@@ -576,8 +690,11 @@ class IMP_Contents extends MIME_Contents {
         $primary_type = $part->getPrimaryType();
         if (($primary_type == MIME::type(TYPEMULTIPART)) ||
             ($primary_type == MIME::type(TYPETEXT))) {
-            $this->_body_id = $this->_findBody($part);
-            return $this->_body_id;
+            $body_id = $this->_findBody($part, $subtype);
+            if ($subtype !== null) {
+                $this->_body_id = $body_id;
+            }
+            return $body_id;
         }
 
         return null;
@@ -586,23 +703,29 @@ class IMP_Contents extends MIME_Contents {
     /**
      * Processes a MIME Part and looks for "body" data.
      *
+     * @todo  Copied to MIME_Contents but kept here for BC.
+     *
      * @access private
      *
      * @return string  The MIME ID of the main "body" part.
      */
-    function _findBody($mime_part)
+    function _findBody($mime_part, $subtype)
     {
         if (intval($mime_part->getMIMEId()) < 2 ||
             $mime_part->getInformation('alternative') === 0) {
             if ($mime_part->getPrimaryType() == MIME::type(TYPEMULTIPART)) {
-                foreach ($mime_part->getParts() as $part) {
-                    if (($partid = $this->_findBody($part))) {
+                $parts = $mime_part->getParts();
+                while ($part = array_shift($parts)) {
+                    if (($partid = $this->_findBody($part, $subtype))) {
                         return $partid;
                     }
                 }
             } elseif ($mime_part->getPrimaryType() == MIME::type(TYPETEXT)) {
-                if ($mime_part->getBytes() ||
-                    $this->getBodyPart($mime_part->getMIMEId())) {
+                if ($mime_part->getDisposition() != 'attachment' &&
+                    (($subtype === null) ||
+                     ($subtype == $mime_part->getSubType())) &&
+                    ($mime_part->getBytes() ||
+                     $this->getBodyPart($mime_part->getMIMEId()))) {
                     return $mime_part->getMIMEId();
                 }
             }
@@ -616,20 +739,21 @@ class IMP_Contents extends MIME_Contents {
      *
      * @access private
      *
-     * @param integer $index  The IMAP index of the current message.
+     * @param mixed $in  Either an index string (see IMP_Contents::singleton()
+     *                   for the format) or a MIME_Message object.
      *
      * @return string  A unique cache ID.
      */
-    function _createCacheID($index = null)
+    function _createCacheID($in = null)
     {
-        if (is_null($index)) {
+        if ($in === null) {
             if (isset($this)) {
-                $index = $this->_index;
+                $in = $this->_index . IMP_IDX_SEP . $this->_mailbox;
             } else {
                 return parent::_createCacheID();
             }
         }
-        return md5(implode('|', array($index, $_SESSION['imp']['thismailbox'])));
+        return md5(serialize($in) . Auth::getAuth());
     }
 
     /**
@@ -637,7 +761,7 @@ class IMP_Contents extends MIME_Contents {
      * data.
      *
      * @access private
-     * @since IMP 3.1
+     * @since IMP 4.1
      *
      * @param MIME_Part &$mime_part  The MIME_Part object to work with.
      * @param boolean $all           Download the entire parts contents?
@@ -650,7 +774,13 @@ class IMP_Contents extends MIME_Contents {
             if ($all && ($mime_part->getType() == 'message/rfc822')) {
                 $id = substr($mime_part->getMIMEId(), 0, -2);
             }
-            $mime_part->setContents($this->getBodyPart($id));
+            $contents = $this->getBodyPart($id);
+            if (($mime_part->getPrimaryType() == 'text') &&
+                (String::upper($mime_part->getCharset()) == 'US-ASCII') &&
+                MIME::is8bit($contents)) {
+                $contents = String::convertCharset($contents, 'US-ASCII');
+            }
+            $mime_part->setContents($contents);
         }
         $mime_part->setInformation('imp_contents_set', true);
     }
@@ -658,7 +788,7 @@ class IMP_Contents extends MIME_Contents {
     /**
      * Fetch part of a MIME message.
      *
-     * @since IMP 3.1
+     * @since IMP 4.1
      *
      * @param integer $id   The MIME ID of the part requested.
      * @param boolean $all  If this is a header part, should we return all text
@@ -708,4 +838,34 @@ class IMP_Contents extends MIME_Contents {
         return ($canonical) ? $part->toCanonicalString() : $part->toString();
     }
 
+    /**
+     * Remove all attachment entries for the given part.
+     * TODO: This is a total hack to tide us over to Horde 4.0.
+     *
+     * @since IMP 4.2
+     *
+     * @param string $index  The index to remove from the attachment list.
+     */
+    function removeAtcEntry($index)
+    {
+        $index = strval($index);
+        foreach (array_keys($this->_atc) as $val) {
+            if ((strpos($val, $index) === 0) && ($val != $index . '.0')) {
+                unset($this->_atc[$val]);
+            }
+        }
+    }
+
+    /**
+     * Return the attachment count.
+     *
+     * @since IMP 4.2
+     *
+     * @param return array  The attachment count.
+     */
+    function attachmentCount()
+    {
+        return count($this->_atc);
+    }
+
 }
