@@ -14,17 +14,18 @@ require_once 'Horde/MIME/Structure.php';
  *   application/x-pkcs7-mime
  *   application/x-pkcs7-signature
  *
- * $Horde: imp/lib/MIME/Viewer/pkcs7.php,v 1.68.2.12 2005/09/30 03:22:37 selsky Exp $
+ * This class may add the following parameters to the URL:
+ *   'smime_verify_msg' -- Do verification of S/MIME signed data.
  *
- * Copyright 2002-2005 Mike Cochrane <mike@graftonhall.co.nz>
- * Copyright 2004-2005 Michael Slusarz <slusarz@curecanti.org>
+ * $Horde: imp/lib/MIME/Viewer/pkcs7.php,v 1.68.2.26 2008/05/08 07:03:23 slusarz Exp $
+ *
+ * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
  *
  * @author  Mike Cochrane <mike@graftonhall.co.nz>
- * @author  Michael Slusarz <slusarz@curecanti.org>
- * @since   IMP 4.0
+ * @author  Michael Slusarz <slusarz@horde.org>
  * @package Horde_MIME_Viewer
  */
 class IMP_MIME_Viewer_pkcs7 extends MIME_Viewer {
@@ -64,6 +65,17 @@ class IMP_MIME_Viewer_pkcs7 extends MIME_Viewer {
      */
     var $_headers;
 
+    /**
+     * Some mailers set S/MIME messages to always be attachments.  However,
+     * most of the time S/MIME is used to secure the contents of the message,
+     * so displaying as an attachment makes no sense.  Therefore, force
+     * viewing inline (or at least let MIME_Viewer/MIME_Contents make the
+     * determination on whether the data can be viewed inline or not).
+     *
+     * @var boolean
+     */
+    var $_forceinline = true;
+
     /**
      * Render out the currently set contents.
      *
@@ -80,7 +92,7 @@ class IMP_MIME_Viewer_pkcs7 extends MIME_Viewer {
         $msg = '';
 
         if (empty($this->_impSmime)) {
-            $this->_impSmime = &new IMP_SMIME();
+            $this->_impSmime = new IMP_SMIME();
         }
 
         /* Check to see if S/MIME support is available. */
@@ -89,24 +101,50 @@ class IMP_MIME_Viewer_pkcs7 extends MIME_Viewer {
             !is_a($openssl_check, 'PEAR_Error')) {
             /* We need to insert JavaScript code now if S/MIME support is
                active. */
-            $msg = Util::bufferOutput('require_once', IMP_TEMPLATES . '/smime/open_smime_win.js');
+            $msg = Util::bufferOutput(array('Horde', 'addScriptFile'), 'prototype.js', 'imp', true);
+            $msg .= Util::bufferOutput(array('Horde', 'addScriptFile'), 'popup.js', 'imp', true);
         }
 
         /* Get the type of message now. */
         $type = $this->_getSMIMEType();
         switch ($type) {
-            case 'signed':
-                $msg .= $this->_outputSMIMESigned();
-                break;
+        case 'signed':
+            $msg .= $this->_outputSMIMESigned();
+            break;
 
-            case 'encrypted':
-                $msg .= $this->_outputSMIMEEncrypted();
-                break;
+        case 'encrypted':
+            $msg .= $this->_outputSMIMEEncrypted();
+            break;
         }
 
         return $msg;
     }
 
+    /**
+     * Generates HTML output for the S/MIME key in
+     * 'application/pkcs7-signature' MIME_Parts.
+     *
+     * @access private
+     *
+     * @return string  The HTML output.
+     */
+    function _outputSMIMEKey()
+    {
+        if (!$GLOBALS['prefs']->getValue('use_smime')) {
+            return _("S/MIME support is not enabled.");
+        } else {
+            $mime = &$this->mime_part;
+            $signenc = $mime->getInformation('smime_signenc');
+            $raw_text = $this->_getRawSMIMEText();
+            if ($signenc && $mime->getInformation('smime_from')) {
+                $smime_from = $mime->getInformation('smime_from');
+                $raw_text = "From: $smime_from\n" . $raw_text;
+            }
+            $sig_result = $this->_impSmime->verifySignature($raw_text);
+            return $this->_impSmime->certToHTML($sig_result->cert);
+        }
+    }
+
     /**
      * Generates HTML output for 'multipart/signed',
      * 'application/pkcs7-signature' and
@@ -118,6 +156,10 @@ class IMP_MIME_Viewer_pkcs7 extends MIME_Viewer {
      */
     function _outputSMIMESigned()
     {
+        if (Util::getFormData('viewkey')) {
+            return $this->_outputSMIMEKey();
+        }
+
         $cert = $text = '';
         $mime = &$this->mime_part;
         $mimetype = $mime->getType();
@@ -159,7 +201,7 @@ class IMP_MIME_Viewer_pkcs7 extends MIME_Viewer {
                 $this->_status[] = _("This message does not appear to be in the correct S/MIME format.");
             }
         } elseif (!$active) {
-            $this->_status[] = _("S/MIME support is not enabled so the contents of this signed message can not be displayed."); 
+            $this->_status[] = _("S/MIME support is not enabled so the contents of this signed message cannot be displayed.");
         }
 
         if ($active) {
@@ -168,7 +210,16 @@ class IMP_MIME_Viewer_pkcs7 extends MIME_Viewer {
                 $smime_from = $mime->getInformation('smime_from');
                 $raw_text = "From: $smime_from\n" . $raw_text;
             }
-            $sig_result = $this->_impSmime->verifySignature($raw_text);
+
+            if ($GLOBALS['prefs']->getValue('smime_verify') ||
+                Util::getFormData('smime_verify_msg')) {
+                $sig_result = $this->_impSmime->verifySignature($raw_text);
+            } elseif (isset($_SESSION['imp']['viewmode']) &&
+                      ($_SESSION['imp']['viewmode'] == 'imp')) {
+                // TODO: Fix to work with DIMP
+                $this->_status[] = Horde::link(Util::addParameter(Horde::selfUrl(true), 'smime_verify_msg', 1)) . _("Click HERE to verify the message.") . '</a>';
+            }
+
             if (!isset($subpart)) {
                 $msg_data = $this->_impSmime->extractSignedContents($raw_text);
                 if (is_a($msg_data, 'PEAR_Error')) {
@@ -180,23 +231,39 @@ class IMP_MIME_Viewer_pkcs7 extends MIME_Viewer {
             }
 
             $text = $this->_outputStatus();
-            $text .= $this->_outputSMIMESignatureTest($sig_result->result, $sig_result->email);
-            if (!empty($sig_result->cert)) {
-                if (!isset($smime_from) && isset($this->_headers['from'])) {
-                    $smime_from = $this->_headers['from'];
-                }
-                if (isset($smime_from) &&
-                    $GLOBALS['registry']->hasMethod('contacts/addField') &&
-                    $GLOBALS['prefs']->getValue('add_source')) {
-                    $this->_status[] = Horde::link('', '', null, null, $this->_impSmime->savePublicKeyURL($sig_result->cert, $smime_from) . ' return false;') . sprintf(_("Click to Save S/MIME certificate of %s in your Address Book"), @htmlspecialchars($smime_from, ENT_COMPAT, NLS::getCharset())) . '</a>.';
-                    $text .= $this->_outputStatus();
+            if ($sig_result !== null) {
+                $text .= $this->_outputSMIMESignatureTest($sig_result->result, $sig_result->email);
+                if (!empty($sig_result->cert)) {
+                    $cert_details = $this->_impSmime->parseCert($sig_result->cert);
+                    if (isset($cert_details['certificate']['subject']['CommonName'])) {
+                        $subject = $cert_details['certificate']['subject']['CommonName'];
+                    } elseif (isset($cert_details['certificate']['subject']['Email'])) {
+                        $subject = $cert_details['certificate']['subject']['Email'];
+                    } elseif (isset($sig_result->email)) {
+                        $subject = $sig_result->email;
+                    } elseif (isset($smime_from)) {
+                        $subject = $smime_from;
+                    } elseif (isset($this->_headers['from'])) {
+                        $subject = $this->_headers['from'];
+                    } else {
+                        $subject = null;
+                    }
+                    if (isset($subpart) &&
+                        !empty($subject) &&
+                        $GLOBALS['registry']->hasMethod('contacts/addField') &&
+                        $GLOBALS['prefs']->getValue('add_source')) {
+                        $this->_status[] = sprintf(_("The S/MIME certificate of %s: "), @htmlspecialchars($subject, ENT_COMPAT, NLS::getCharset())) .
+                            $this->_contents->linkViewJS($subpart, 'view_attach', _("View"), '', null, array('viewkey' => 1)) . '/' .
+                            Horde::link('#', '', null, null, $this->_impSmime->savePublicKeyURL($sig_result->cert) . ' return false;') . _("Save in your Address Book") . '</a>';
+                        $text .= $this->_outputStatus();
+                    }
                 }
             }
         }
 
         if (isset($mime_message)) {
             /* We need to stick the output into a MIME_Contents object. */
-            $mc = &new MIME_Contents($mime_message, array('download' => 'download_attach', 'view' => 'view_attach'), array(&$this->_contents));
+            $mc = new MIME_Contents($mime_message, array('download' => 'download_attach', 'view' => 'view_attach'), array(&$this->_contents));
             $mc->buildMessage();
 
             $text .= '<table cellpadding="0" cellspacing="0">' . $mc->getMessage(true) . '</table>';
@@ -231,13 +298,22 @@ class IMP_MIME_Viewer_pkcs7 extends MIME_Viewer {
             return $this->_outputStatus();
         }
 
+        if (!$this->_impSmime->getPersonalPrivateKey()) {
+            $this->_status[] = _("No personal private key exists so the message is unable to be decrypted.");
+            return $this->_outputStatus();
+        }
+
         /* Make sure we have a passphrase. */
         $passphrase = $this->_impSmime->getPassphrase();
         if ($passphrase === false) {
-            $url = $this->_impSmime->getJSOpenWinCode('open_passphrase_dialog');
-            $this->_status[] = Horde::link('', _("You must enter the passphrase for your S/MIME private key to view this message"), null, null, $url . ' return false;') . '<i>' . _("You must enter the passphrase for your S/MIME private key to view this message") . '</i></a>.';
-            $msg .= $this->_outputStatus() .
-                '<script language="JavaScript" type="text/javascript">' . $url . ';</script>';
+            if (isset($_SESSION['imp']['viewmode']) &&
+                ($_SESSION['imp']['viewmode'] == 'imp')) {
+                // TODO: Fix to work with DIMP
+                $url = $this->_impSmime->getJSOpenWinCode('open_passphrase_dialog');
+                $this->_status[] = Horde::link('#', _("You must enter the passphrase for your S/MIME private key to view this message"), null, null, $url . ' return false;') . '<em>' . _("You must enter the passphrase for your S/MIME private key to view this message") . '</em></a>.';
+                $msg .= $this->_outputStatus() .
+                    '<script type="text/javascript">' . $url . ';</script>';
+            }
             return $msg;
         }
 
@@ -265,7 +341,7 @@ class IMP_MIME_Viewer_pkcs7 extends MIME_Viewer {
             }
 
             /* We need to stick the output into a MIME_Contents object. */
-            $mc = &new MIME_Contents($mime_message, array('download' => 'download_attach', 'view' => 'view_attach'), array(&$this->_contents));
+            $mc = new MIME_Contents($mime_message, array('download' => 'download_attach', 'view' => 'view_attach'), array(&$this->_contents));
             $mc->buildMessage();
             $msg .= '<table cellpadding="0" cellspacing="0">' . $mc->getMessage(true) . '</table>';
         } else {
@@ -327,8 +403,8 @@ class IMP_MIME_Viewer_pkcs7 extends MIME_Viewer {
     /* Various formatting helper functions. */
     function _initStatus($src, $alt = '')
     {
-        if (is_null($this->_icon)) {
-            $this->_icon = Horde::img($src, $alt, 'height="16" width="16"', '') . '&nbsp;';
+        if ($this->_icon === null) {
+            $this->_icon = Horde::img($src, $alt, 'height="16" width="16"', '');
         }
     }
 
@@ -370,6 +446,7 @@ class IMP_MIME_Viewer_pkcs7 extends MIME_Viewer {
             /* This message has been verified but there was no output
                from the PGP program. */
             if (empty($result) || ($result === true)) {
+               $email = (is_array($email)) ? implode(', ', $email): $email;
                $result = sprintf(_("The message has been verified. Sender: %s."), htmlspecialchars($email));
             }
         }
@@ -429,10 +506,10 @@ class IMP_MIME_Viewer_pkcs7 extends MIME_Viewer {
         }
 
         switch ($type) {
-            case 'multipart/signed':
-            case 'application/pkcs7-signature':
-            case 'application/x-pkcs7-signature':
-                return 'signed';
+        case 'multipart/signed':
+        case 'application/pkcs7-signature':
+        case 'application/x-pkcs7-signature':
+            return 'signed';
         }
     }
 
