@@ -5,15 +5,14 @@ require_once 'Horde/MIME/Viewer/tnef.php';
 /**
  * The IMP_MIME_Viewer_tnef class allows MS-TNEF attachments to be displayed.
  *
- * $Horde: imp/lib/MIME/Viewer/tnef.php,v 1.25.10.5 2005/07/03 03:42:45 selsky Exp $
+ * $Horde: imp/lib/MIME/Viewer/tnef.php,v 1.25.10.12 2008/09/04 12:34:54 jan Exp $
  *
- * Copyright 2002-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
  *
- * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
- * @since   IMP 4.0
+ * @author  Michael Slusarz <slusarz@horde.org>
  * @package Horde_MIME_Viewer
  */
 class IMP_MIME_Viewer_tnef extends MIME_Viewer_tnef {
@@ -81,7 +80,9 @@ class IMP_MIME_Viewer_tnef extends MIME_Viewer_tnef {
         $text = '';
 
         /* Make sure the contents are in the MIME_Part object. */
-        $this->mime_part->setContents($contents->getBodyPart($this->mime_part->getMIMEId()));
+        if (!$this->mime_part->getContents()) {
+            $this->mime_part->setContents($contents->getBodyPart($this->mime_part->getMIMEId()));
+        }
 
         /* Get the data from the attachment. */
         $tnefData = $this->_getSubparts();
@@ -94,7 +95,7 @@ class IMP_MIME_Viewer_tnef extends MIME_Viewer_tnef {
                 $temp_part = $this->mime_part;
                 $temp_part->setName($data['name']);
                 $temp_part->setDescription($data['name']);
- 
+
                 /* Short-circuit MIME-type guessing for winmail.dat parts;
                    we're showing enough entries for them already. */
                 require_once 'Horde/MIME/Magic.php';
@@ -105,7 +106,7 @@ class IMP_MIME_Viewer_tnef extends MIME_Viewer_tnef {
                 }
                 $temp_part->setType($type);
 
-                $link = $contents->linkView($temp_part, 'view_attach', $data['name'], array('jstext' => sprintf(_("View %s"), $data['name']), 'viewparams' => array('tnef_attachment' => ($key + 1))));
+                $link = $contents->linkView($temp_part, 'view_attach', htmlspecialchars($data['name']), array('jstext' => sprintf(_("View %s"), $data['name']), 'viewparams' => array('tnef_attachment' => ($key + 1))));
                 $text .= _("Attached File:") . '&nbsp;&nbsp;' . $link . '&nbsp;&nbsp;(' . $data['type'] . '/' . $data['subtype'] . ")<br />\n";
             }
         }
