@@ -3,15 +3,14 @@
  * IMP_Quota:: provides an API for retrieving Quota details from a mail
  * server.
  *
- * $Horde: imp/lib/Quota.php,v 1.23.10.9 2007/01/02 13:54:56 jan Exp $
+ * $Horde: imp/lib/Quota.php,v 1.23.10.14 2008/07/03 13:06:16 jan Exp $
  *
- * Copyright 2002-2007 Mike Cochrane <mike@graftonhall.co.nz>
+ * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
  *
  * @author  Mike Cochrane <mike@graftonhall.co.nz>
- * @since   IMP 4.0
  * @package IMP_Quota
  */
 class IMP_Quota {
@@ -23,63 +22,6 @@ class IMP_Quota {
      */
     var $_params = array();
 
-    /**
-     * Attempts to return a concrete Quota instance based on $driver.
-     *
-     * @param string $driver  The type of concrete Quota subclass to return.
-     * @param array $params   A hash containing any additional configuration or
-     *                        connection parameters a subclass might need.
-     *
-     * @return mixed  The newly created concrete Quota instance, or false
-     *                on error.
-     */
-    function &factory($driver, $params = array())
-    {
-        $driver = basename($driver);
-        require_once dirname(__FILE__) . '/Quota/' . $driver . '.php';
-        $class = 'IMP_Quota_' . $driver;
-        if (class_exists($class)) {
-            $quota = &new $class($params);
-        } else {
-            $quota = false;
-        }
-
-        return $quota;
-    }
-
-    /**
-     * Attempts to return a reference to a concrete Quota instance based on
-     * $driver.
-     *
-     * It will only create a new instance if no Quota instance with the same
-     * parameters currently exists.
-     *
-     * This should be used if multiple quota sources are required.
-     *
-     * This method must be invoked as: $var = &Quota::singleton()
-     *
-     * @param string $driver  The type of concrete Quota subclass to return.
-     * @param array $params   A hash containing any additional configuration or
-     *                        connection parameters a subclass might need.
-     *
-     * @return mixed  The created concrete Quota instance, or false on error.
-     */
-    function &singleton($driver, $params = array())
-    {
-        static $instances;
-
-        if (!isset($instances)) {
-            $instances = array();
-        }
-
-        $signature = serialize(array($driver, $params));
-        if (!isset($instances[$signature])) {
-            $instances[$signature] = &IMP_Quota::factory($driver, $params);
-        }
-
-        return $instances[$signature];
-    }
-
     /**
      * Constructor.
      *
@@ -109,4 +51,79 @@ class IMP_Quota {
         return array('usage' => 0, 'limit' => 0);
     }
 
+    /**
+     * Returns the quota messages variants, including sprintf placeholders.
+     *
+     * @return array  A hash with quota message templates.
+     */
+    function getMessages()
+    {
+        return array(
+            'long' => isset($this->_params['format']['long'])
+                ? $this->_params['format']['long']
+                : _("Quota status: %.2f MB / %.2f MB  (%.2f%%)"),
+            'short' => isset($this->_params['format']['short'])
+                ? $this->_params['format']['short']
+                : _("%.0f%% of %.0f MB"),
+            'nolimit_long' => isset($this->_params['format']['nolimit_long'])
+                ? $this->_params['format']['nolimit_long']
+                : _("Quota status: %.2f MB / NO LIMIT"),
+            'nolimit_short' => isset($this->_params['format']['nolimit_short'])
+                ? $this->_params['format']['nolimit_short']
+                : _("%.0f MB"));
+    }
+
+    /**
+     * Attempts to return a concrete Quota instance based on $driver.
+     *
+     * @param string $driver  The type of concrete Quota subclass to return.
+     * @param array $params   A hash containing any additional configuration or
+     *                        connection parameters a subclass might need.
+     *
+     * @return mixed  The newly created concrete Quota instance, or false
+     *                on error.
+     */
+    function &factory($driver, $params = array())
+    {
+        $driver = basename($driver);
+        require_once dirname(__FILE__) . '/Quota/' . $driver . '.php';
+        $class = 'IMP_Quota_' . $driver;
+        if (class_exists($class)) {
+            $quota = new $class($params);
+        } else {
+            $quota = false;
+        }
+
+        return $quota;
+    }
+
+    /**
+     * Attempts to return a reference to a concrete Quota instance based on
+     * $driver.
+     *
+     * It will only create a new instance if no Quota instance with the same
+     * parameters currently exists.
+     *
+     * This should be used if multiple quota sources are required.
+     *
+     * This method must be invoked as: $var = &Quota::singleton()
+     *
+     * @param string $driver  The type of concrete Quota subclass to return.
+     * @param array $params   A hash containing any additional configuration or
+     *                        connection parameters a subclass might need.
+     *
+     * @return mixed  The created concrete Quota instance, or false on error.
+     */
+    function &singleton($driver, $params = array())
+    {
+        static $instances = array();
+
+        $signature = serialize(array($driver, $params));
+        if (!isset($instances[$signature])) {
+            $instances[$signature] = &IMP_Quota::factory($driver, $params);
+        }
+
+        return $instances[$signature];
+    }
+
 }
