@@ -15,9 +15,9 @@
  *                   partition label.  By default, quota will determine
  *                   quota information using the user's home directory value.
  *
- * $Horde: imp/lib/Quota/command.php,v 1.11.10.13 2007/01/02 13:55:02 jan Exp $
+ * $Horde: imp/lib/Quota/command.php,v 1.11.10.16 2008/07/02 09:31:14 jan Exp $
  *
- * Copyright 2002-2007 Eric Jon Rostetter <eric.rostetter@physics.utexas.edu>
+ * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
@@ -34,13 +34,11 @@ class IMP_Quota_command extends IMP_Quota {
      */
     function IMP_Quota_command($params = array())
     {
-        $this->_params = array(
-            'quota_path' => 'quota',
-            'grep_path'  => 'grep',
-            'partition'  => null
-        );
-
-        $this->_params = array_merge($this->_params, $params);
+        $params = array_merge(array('quota_path' => 'quota',
+                                    'grep_path'  => 'grep',
+                                    'partition'  => null),
+                              $params);
+        parent::IMP_Quota($params);
     }
 
     /**
