@@ -1,17 +1,18 @@
 <?php
-
-define('IMP_PREF_NO_FOLDER', '**nofolder');
-
 /**
- * $Horde: imp/lib/prefs.php,v 1.3.10.22 2007/11/23 17:44:46 jan Exp $
+ * $Horde: imp/lib/prefs.php,v 1.3.10.31 2008/09/17 12:22:07 jan Exp $
  *
- * Copyright 1999-2007 Charles J. Hagenbuch <chuck@horde.org>
- * Copyright 1999-2007 Jon Parise <jon@horde.org>
+ * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
  *
- * See the enclosed file COPYING for license information (GPL).  If you
+ * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
+ *
+ * @package Horde_Prefs
  */
 
+define('IMP_PREF_NO_FOLDER', '%nofolder');
+define('IMP_PREF_VTRASH', '%vtrash');
+
 function handle_sentmailselect($updated)
 {
     global $conf, $prefs, $identity;
@@ -77,7 +78,29 @@ function handle_folderselect($updated)
 
 function handle_trashselect($updated)
 {
-    return $updated | handlefolders($updated, 'trash_folder', 'trash', 'trash_new');
+    global $prefs;
+    $ret = true;
+
+    if (Util::getFormData('trash') == IMP_PREF_VTRASH) {
+        if ($prefs->isLocked('use_vtrash')) {
+            $ret = false;
+        } else {
+            $prefs->setValue('use_vtrash', 1);
+            $prefs->setValue('trash_folder', '');
+        }
+    } else {
+        if ($prefs->isLocked('trash_folder')) {
+            $ret = false;
+        } else {
+            $ret = $updated | handlefolders($updated, 'trash_folder', 'trash', 'trash_new');
+            if ($ret) {
+                $prefs->setValue('use_vtrash', 0);
+                $prefs->setDirty('trash_folder', true);
+            }
+        }
+    }
+
+    return $ret;
 }
 
 function handle_sourceselect($updated)
@@ -85,19 +108,19 @@ function handle_sourceselect($updated)
     global $prefs;
 
     $search_sources = Util::getFormData('search_sources');
-    if (!is_null($search_sources)) {
+    if ($search_sources !== null) {
         $prefs->setValue('search_sources', $search_sources);
         $updated = true;
     }
 
     $search_fields_string = Util::getFormData('search_fields_string');
-    if (!is_null($search_fields_string)) {
+    if ($search_fields_string !== null) {
         $prefs->setValue('search_fields', $search_fields_string);
         $updated = true;
     }
 
     $add_source = Util::getFormData('add_source');
-    if (!is_null($add_source)) {
+    if ($add_source !== null) {
         $prefs->setValue('add_source', $add_source);
         $updated = true;
     }
@@ -131,62 +154,76 @@ function handle_defaultsearchselect($updated)
     return true;
 }
 
+function handle_soundselect($updated)
+{
+    return $GLOBALS['prefs']->setValue('nav_audio', Util::getFormData('nav_audio'));
+}
+
 function prefs_callback()
 {
     global $prefs;
 
-    /* Never allow trash and Virtual Trash to be activated at the same time.
-     * If they collide, and a trash folder is defined, always default to trash
-     * folder. */
-    if ($prefs->isDirty('use_vtrash') || $prefs->isDirty('use_trash')) {
-        if ($prefs->getValue('use_vtrash') && $prefs->getValue('use_trash')) {
-            if ($prefs->getValue('trash_folder')) {
-                $prefs->setValue('use_vtrash', 0);
-                $GLOBALS['notification']->push(_("Cannot activate both a Trash folder and Virtual Trash. A Trash folder will be used."), 'horde.error');
-            } else {
-                $prefs->setValue('use_trash', 0);
-                $GLOBALS['notification']->push(_("Cannot activate both a Trash folder and Virtual Trash. A Virtual Trash folder will be used."), 'horde.error');
-            }
-        }
-    }
-
     /* Always check to make sure we have a valid trash folder if delete to
      * trash is active. */
     if (($prefs->isDirty('use_trash') || $prefs->isDirty('trash_folder')) &&
         $prefs->getValue('use_trash') &&
-        !$prefs->getValue('trash_folder')) {
-        $prefs->setValue('use_trash', 0);
-        $GLOBALS['notification']->push(_("Cannot delete to trash unless a Trash folder is defined."), 'horde.error');
+        !$prefs->getValue('trash_folder') &&
+        !$prefs->getValue('use_vtrash')) {
+        $GLOBALS['notification']->push(_("You have activated move to Trash but no Trash folder is defined. You will be unable to delete messages until you set a Trash folder in the preferences."), 'horde.warning');
     }
 
-    if (($prefs->isDirty('use_vtrash') && $prefs->getValue('use_vtrash')) ||
-        $GLOBALS['prefs']->isDirty('use_vinbox')) {
+    if ($prefs->isDirty('use_vtrash') || $prefs->isDirty('use_vinbox')) {
         require_once IMP_BASE . '/lib/Search.php';
         $imp_search = new IMP_Search();
-        $imp_search->sessionSetup();
+        $imp_search->sessionSetup(true);
+
+        if ($prefs->isDirty('use_vtrash')) {
+            require_once IMP_BASE . '/lib/IMAP/Cache.php';
+            require_once IMP_BASE . '/lib/IMAP/Tree.php';
+            $imap_cache = &IMP_IMAP_Cache::singleton();
+            $imap_tree = &IMP_Tree::singleton();
+            foreach ($imap_tree->folderList(IMPTREE_FLIST_UNSUB) as $val) {
+                $imap_cache->expireCache($val, 1 | 2 | 4);
+            }
+        }
     }
 
-    if ($GLOBALS['prefs']->isDirty('subscribe')) {
+    if ($prefs->isDirty('subscribe') || $prefs->isDirty('tree_view')) {
+        require_once IMP_BASE . '/lib/Folder.php';
         require_once IMP_BASE . '/lib/IMAP/Tree.php';
+        $imp_folder = &IMP_Folder::singleton();
+        $imp_folder->clearFlistCache();
         $imptree = &IMP_Tree::singleton();
         $imptree->init();
     }
 
     /* If a maintenance option has been activated, we need to make sure the
      * global Horde 'do_maintenance' pref is also active. */
-    if (!$GLOBALS['prefs']->isLocked('do_maintenance') &&
-        !$GLOBALS['prefs']->getValue('do_maintenance')) {
+    if (!$prefs->isLocked('do_maintenance') &&
+        !$prefs->getValue('do_maintenance')) {
         foreach (array('rename_sentmail_monthly', 'delete_sentmail_monthly', 'purge_sentmail', 'delete_attachments_monthly', 'purge_trash') as $val) {
-            if ($GLOBALS['prefs']->getValue($val)) {
-                $GLOBALS['prefs']->setValue('do_maintenance', true);
+            if ($prefs->getValue($val)) {
+                $prefs->setValue('do_maintenance', true);
                 break;
             }
         }
     }
+
+    if ($prefs->isDirty('mail_domain')) {
+        $maildomain = preg_replace('/[^-\.a-z0-9]/i', '', $prefs->getValue('mail_domain'));
+        $prefs->setValue('maildomain', $maildomain);
+        if (!empty($maildomain)) {
+            $_SESSION['imp']['maildomain'] = $maildomain;
+        }
+    }
+
+    if ($prefs->isDirty('compose_popup')) {
+        $GLOBALS['notification']->push('if (window.parent.frames.horde_menu) window.parent.frames.horde_menu.location.reload();', 'javascript');
+    }
 }
 
 require_once IMP_BASE . '/lib/Maintenance/imp.php';
-$maint = new Maintenance_IMP();
+$maint = &new Maintenance_IMP();
 foreach (($maint->exportIntervalPrefs()) as $val) {
     $$val = &$intervals;
 }
@@ -196,3 +233,10 @@ if (!$GLOBALS['registry']->call('mail/getStream')) {
     header('Location: ' . Util::addParameter(Horde::applicationUrl('redirect.php'), 'url', Horde::selfUrl(true)));
     exit;
 }
+
+// BC check: check for fckeditor (Horde 3.2).
+require_once 'Horde/Editor.php';
+if (!is_callable(array('Horde_Editor', 'availableEditors')) ||
+    !in_array('fckeditor', Horde_Editor::availableEditors())) {
+    $GLOBALS['_prefs']['jseditor']['enum'] = array('xinha' => _("Xinha"));;
+}
